"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = getMockMiddleware;

var _path = require("path");

var _bodyParser = _interopRequireDefault(require("body-parser"));

var _glob = _interopRequireDefault(require("glob"));

var _assert = _interopRequireDefault(require("assert"));

var _chokidar = _interopRequireDefault(require("chokidar"));

var _pathToRegexp = require("path-to-regexp");

var _signale = _interopRequireDefault(require("signale"));

var _multer = _interopRequireDefault(require("multer"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

// const VALID_METHODS = ['get', 'post', 'put', 'patch', 'delete', 'head'];
var BODY_PARSED_METHODS = ['post', 'put', 'patch', 'delete'];

var debug = require('debug')('mock-api');

function getMockMiddleware(mockDir, options) {
  var absMockPath = mockDir;
  var errors = [];
  var mockData = getConfig();
  watch();

  function watch() {
    if (process.env.WATCH_FILES === 'none') return;

    var watcher = _chokidar["default"].watch([absMockPath], {
      ignoreInitial: true
    });

    watcher.on('all', function (event, file) {
      debug("[".concat(event, "] ").concat(file, ", reload mock data"));
      mockData = getConfig();

      if (!errors.length) {
        _signale["default"].success("Mock file parse success");
      }
    });
  }

  function getConfig() {
    // Clear errors
    errors.splice(0, errors.length);
    cleanRequireCache();
    var ret = {};

    var mockFiles = _glob["default"].sync('**/*.[jt]s', {
      cwd: absMockPath,
      ignore: (options || {}).ignore || []
    }).map(function (p) {
      return (0, _path.join)(absMockPath, p);
    });

    debug("load mock data from ".concat(absMockPath, ", including files ").concat(JSON.stringify(mockFiles)));

    try {
      ret = mockFiles.reduce(function (memo, mockFile) {
        var m = require(mockFile); // eslint-disable-line


        memo = _objectSpread(_objectSpread({}, memo), m["default"] || m);
        return memo;
      }, {});
    } catch (e) {
      errors.push(e);

      _signale["default"].error("Mock file parse failed: ".concat(e.message));
    }

    return normalizeConfig(ret);
  }

  function parseKey(key) {
    var method = 'get';
    var path = key;

    if (key.indexOf(' ') > -1) {
      var splited = key.split(' ');
      method = splited[0].toLowerCase();
      path = splited[1]; // eslint-disable-line
    } // debug(`parsed: ${method} ${path}`);
    // assert(
    //   VALID_METHODS.includes(method),
    //   `Invalid method ${method} for path ${path}, please check your mock files.`
    // );


    return {
      method: method,
      path: path
    };
  }

  function createHandler(method, path, handler) {
    return function (req, res, next) {
      if (BODY_PARSED_METHODS.includes(method)) {
        _bodyParser["default"].json({
          limit: '5mb',
          strict: false
        })(req, res, function () {
          _bodyParser["default"].urlencoded({
            limit: '5mb',
            extended: true
          })(req, res, function () {
            sendData();
          });
        });
      } else {
        sendData();
      }

      function sendData() {
        if (typeof handler === 'function') {
          (0, _multer["default"])().any()(req, res, function () {
            handler(req, res, next);
          });
        } else {
          res.json(handler);
        }
      }
    };
  }

  function normalizeConfig(config) {
    return Object.keys(config).reduce(function (memo, key) {
      var handler = config[key];

      var type = _typeof(handler);

      (0, _assert["default"])(type === 'function' || type === 'object', "mock value of ".concat(key, " should be function or object, but got ").concat(type));

      var _parseKey = parseKey(key),
          method = _parseKey.method,
          path = _parseKey.path;

      var keys = [];
      var pathOptions = {
        whitelist: ['%'] // treat %3A as regular chars

      };
      var re = (0, _pathToRegexp.pathToRegexp)(path, keys, pathOptions); // debug(re);

      memo.push({
        method: method,
        path: path,
        re: re,
        keys: keys,
        handler: createHandler(method, path, handler)
      });
      return memo;
    }, []);
  }

  function cleanRequireCache() {
    Object.keys(require.cache).forEach(function (file) {
      if (file.indexOf(absMockPath) > -1) {
        delete require.cache[file];
      }
    });
  }

  function matchMock(req) {
    var targetPath = req.path;
    var targetMethod = req.method.toLowerCase(); // debug(`${targetMethod} ${targetPath}`);

    var _iterator = _createForOfIteratorHelper(mockData),
        _step;

    try {
      for (_iterator.s(); !(_step = _iterator.n()).done;) {
        var mock = _step.value;
        var method = mock.method,
            re = mock.re,
            keys = mock.keys;

        if (method === targetMethod) {
          var match = re.exec(targetPath);

          if (match) {
            var params = {};

            for (var i = 1; i < match.length; i += 1) {
              var key = keys[i - 1];
              var prop = key.name;
              var val = decodeParam(match[i]);

              if (val !== undefined || !hasOwnProperty.call(params, prop)) {
                params[prop] = val;
              }
            }

            req.params = params;
            return mock;
          }
        }
      }
    } catch (err) {
      _iterator.e(err);
    } finally {
      _iterator.f();
    }

    function decodeParam(val) {
      if (typeof val !== 'string' || val.length === 0) {
        return val;
      }

      try {
        return decodeURIComponent(val);
      } catch (err) {
        if (err instanceof URIError) {
          err.message = "Failed to decode param ' ".concat(val, " '");
          err.status = 400;
          err.statusCode = 400;
        }

        throw err;
      }
    }

    return mockData.filter(function (_ref) {
      var method = _ref.method,
          re = _ref.re;
      return method === targetMethod && re.test(targetPath);
    })[0];
  }

  return function mockMiddleware(req, res, next) {
    var match = matchMock(req);

    if (match) {
      debug("mock matched: [".concat(match.method, "] ").concat(match.path));
      return match.handler(req, res, next);
    } else {
      return next();
    }
  };
}